@@ -19,7 +19,8 @@
  */
 
 #include "gtkhtml-editor-private.h"
-#include "gtkhtml-stream.h"
+
+#include <libgnome/gnome-url.h>
 
 /******************************************************************************
  * Action Group Quick Reference
@@ -38,6 +39,7 @@
  * "find"
  * "find-again"
  * "find-and-replace"
+ * "html-mode"		(toggle)
  * "indent"
  * "insert-html-file"
  * "insert-smiley-1"
@@ -56,8 +58,6 @@
  * "justify-center"	(radio)
  * "justify-left"	(radio)
  * "justify-right"	(radio)
- * "mode-html"		(radio)
- * "mode-plain"		(radio)
  * "paste"
  * "paste-quote"
  * "redo"
@@ -86,13 +86,13 @@
  * Core Actions (HTML Only)
  * ------------------------
  * "bold"		(toggle)
+ * "properties-page"
  * "insert-image"
  * "insert-link"
  * "insert-rule"
  * "insert-table"
  * "italic"		(toggle)
  * "monospaced"		(toggle)
- * "properties-page"
  * "size-minus-one"	(radio)
  * "size-minus-two"	(radio)
  * "size-plus-four"	(radio)
@@ -140,98 +140,161 @@ insert_image_response_cb (GtkFileChooser *file_chooser,
                           gint response,
                           GtkhtmlEditor *editor)
 {
+	HTMLObject *image;
+	GtkHTML *html;
 	gchar *uri;
 
 	if (response != GTK_RESPONSE_OK)
 		return;
 
+	html = gtkhtml_editor_get_html (editor);
 	uri = gtk_file_chooser_get_uri (file_chooser);
-	gtkhtml_editor_insert_image (editor, uri);
+	image = html_image_new (
+		html_engine_get_image_factory (html->engine), uri,
+		NULL, NULL, 0, 0, 0, 0, 0, NULL, HTML_VALIGN_NONE, FALSE);
+	html_engine_paste_object (html->engine, image, 1);
 	g_free (uri);
 }
 
 static void
-insert_html_file_ready_cb (GFile *file,
-                           GAsyncResult *result,
-                           GtkhtmlEditor *editor)
+insert_html_file_response_cb (GtkFileChooser *file_chooser,
+                              gint response,
+                              GtkhtmlEditor *editor)
 {
-	GtkHTML *html;
-	GtkHTML *new_html;
-	GtkHTMLStream *stream;
-	GtkWidget *dialog;
+	gchar *filename;
 	gchar *contents;
 	gsize length;
 	GError *error = NULL;
 
-	g_file_load_contents_finish (
-		file, result, &contents, &length, NULL, &error);
-	if (error != NULL)
-		goto fail;
+	if (response != GTK_RESPONSE_OK)
+		return;
 
-	html = gtkhtml_editor_get_html (editor);
-	new_html = GTK_HTML (gtk_html_new ());
+	filename = gtk_file_chooser_get_filename (file_chooser);
 
-	stream = gtk_html_begin (new_html);
-	gtk_html_write (html, stream, contents, length);
-	/* 'stream' is destroyed inside gtk_html_end */
-	gtk_html_end (html, stream, GTK_HTML_STREAM_OK);
+	/* Assume the file encoding is UTF-8. */
+	gtkhtml_editor_get_file_contents (
+		filename, NULL, &contents, &length, &error);
 
-	g_free (contents);
+	/* If we get a conversion error, look for a charset and try again. */
+	if (error != NULL && g_error_matches (error, G_CONVERT_ERROR,
+		G_CONVERT_ERROR_ILLEGAL_SEQUENCE)) {
 
-	html = gtkhtml_editor_get_html (editor);
-	/* 'new_html' destroys gtk_html_insert_gtk_html itself */
-	gtk_html_insert_gtk_html (html, new_html);
+		gchar *charset;
 
-	goto exit;
+		charset = gtkhtml_editor_get_file_charset (filename);
+		if (charset != NULL) {
+			g_clear_error (&error);
+			gtkhtml_editor_get_file_contents (
+				filename, charset, &contents, &length, &error);
+			g_free (charset);
+		}
+	}
 
-fail:
-	dialog = gtk_message_dialog_new (
-		GTK_WINDOW (editor), 0, GTK_MESSAGE_ERROR,
-		GTK_BUTTONS_CLOSE, _("Failed to insert HTML file."));
-	gtk_message_dialog_format_secondary_text (
-		GTK_MESSAGE_DIALOG (dialog), "%s.", error->message);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	if (error == NULL) {
+		GtkHTML *html;
+		GtkHTMLStream *stream;
 
-	g_error_free (error);
+		html = GTK_HTML (gtk_html_new ());
+		stream = gtk_html_begin_content (
+			html, "text/html; charset=utf-8");
+		gtk_html_write (html, stream, contents, length);
+		gtk_html_end (html, stream, GTK_HTML_STREAM_OK);
+		gtk_html_insert_gtk_html (
+			gtkhtml_editor_get_html (editor), html);
+		g_free (contents);
+	} else {
+		/* XXX Show an error dialog. */
+		g_warning ("%s", error->message);
+		g_error_free (error);
+	}
 
-exit:
-	g_object_unref (editor);
+	g_free (filename);
 }
 
 static void
-insert_text_file_ready_cb (GFile *file,
-                           GAsyncResult *result,
-                           GtkhtmlEditor *editor)
+insert_emoticon (GtkhtmlEditor *editor,
+                 GtkAction *action,
+                 const gchar *alt)
 {
 	GtkHTML *html;
-	GtkWidget *dialog;
+	HTMLObject *image;
+	GtkIconInfo *icon_info;
+	const gchar *filename;
+	gchar *icon_name;
+	gchar *uri = NULL;
+
+	html = gtkhtml_editor_get_html (editor);
+
+	g_object_get (action, "icon-name", &icon_name, NULL);
+	icon_info = gtk_icon_theme_lookup_icon (
+		gtk_icon_theme_get_default (), icon_name, 16, 0);
+	g_free (icon_name);
+	g_return_if_fail (icon_info != NULL);
+
+	filename = gtk_icon_info_get_filename (icon_info);
+	if (filename != NULL)
+		uri = g_filename_to_uri (filename, NULL, NULL);
+	gtk_icon_info_free (icon_info);
+	g_return_if_fail (uri != NULL);
+
+	image = html_image_new (
+		html_engine_get_image_factory (html->engine),
+		uri, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
+		HTML_VALIGN_MIDDLE, FALSE);
+	html_image_set_alt (HTML_IMAGE (image), alt);
+	html_engine_paste_object (
+		html->engine, image, html_object_get_length (image));
+
+	g_free (uri);
+}
+
+static void
+insert_text_file_response_cb (GtkFileChooser *file_chooser,
+                              gint response,
+                              GtkhtmlEditor *editor)
+{
 	gchar *contents;
+	gchar *filename;
 	gsize length;
 	GError *error = NULL;
 
-	g_file_load_contents_finish (
-		file, result, &contents, &length, NULL, &error);
-	if (error != NULL)
-		goto fail;
+	if (response != GTK_RESPONSE_OK)
+		return;
 
-	html = gtkhtml_editor_get_html (editor);
-	html_engine_paste_text (html->engine, contents, length);
-	g_free (contents);
+	filename = gtk_file_chooser_get_filename (file_chooser);
 
-	goto exit;
+	/* Assume the file encoding is UTF-8. */
+	gtkhtml_editor_get_file_contents (
+		filename, NULL, &contents, &length, &error);
 
-fail:
-	dialog = gtk_message_dialog_new (
-		GTK_WINDOW (editor), 0, GTK_MESSAGE_ERROR,
-		GTK_BUTTONS_CLOSE, _("Failed to insert text file."));
-	gtk_message_dialog_format_secondary_text (
-		GTK_MESSAGE_DIALOG (dialog), "%s.", error->message);
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	/* If we get a conversion error, look for a charset and try again. */
+	if (error != NULL && g_error_matches (error, G_CONVERT_ERROR,
+		G_CONVERT_ERROR_ILLEGAL_SEQUENCE)) {
 
-exit:
-	g_object_unref (editor);
+		gchar *charset;
+
+		charset = gtkhtml_editor_get_file_charset (filename);
+		if (charset != NULL) {
+			g_clear_error (&error);
+			gtkhtml_editor_get_file_contents (
+				filename, charset, &contents, &length, &error);
+			g_free (charset);
+		}
+	}
+
+	if (error == NULL) {
+		GtkHTML *html;
+
+		html = gtkhtml_editor_get_html (editor);
+		html_engine_paste_text (html->engine, contents, length);
+		g_free (contents);
+	} else {
+		/* XXX Show an error dialog. */
+		g_warning ("%s", error->message);
+		g_error_free (error);
+	}
+
+	g_free (filename);
 }
 
 static void
@@ -265,9 +328,6 @@ action_bold_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	if (gtk_toggle_action_get_active (action))
 		command = "bold-on";
 	else
@@ -557,6 +617,69 @@ action_find_and_replace_cb (GtkAction *action,
 		FALSE, replace_ask_cb, editor);
 }
 
+static void
+action_html_mode_cb (GtkToggleAction *action,
+                     GtkhtmlEditor *editor)
+{
+	GtkActionGroup *action_group;
+	HTMLPainter *new_painter;
+	HTMLPainter *old_painter;
+	GtkHTML *html;
+	gboolean active;
+
+	html = gtkhtml_editor_get_html (editor);
+	active = gtk_toggle_action_get_active (action);
+
+	action_group = editor->priv->html_actions;
+	gtk_action_group_set_sensitive (action_group, active);
+
+	action_group = editor->priv->html_context_actions;
+	gtk_action_group_set_visible (action_group, active);
+
+        gtk_widget_set_sensitive (editor->priv->color_combo_box, active);
+
+	/* Certain paragraph styles are HTML-only. */
+	gtk_action_set_sensitive (ACTION (STYLE_H1), active);
+	gtk_action_set_sensitive (ACTION (STYLE_H2), active);
+	gtk_action_set_sensitive (ACTION (STYLE_H3), active);
+	gtk_action_set_sensitive (ACTION (STYLE_H4), active);
+	gtk_action_set_sensitive (ACTION (STYLE_H5), active);
+	gtk_action_set_sensitive (ACTION (STYLE_H6), active);
+	gtk_action_set_sensitive (ACTION (STYLE_ADDRESS), active);
+
+	/* Swap painters. */
+
+	if (active) {
+		new_painter = editor->priv->html_painter;
+		old_painter = editor->priv->plain_painter;
+	} else {
+		new_painter = editor->priv->plain_painter;
+		old_painter = editor->priv->html_painter;
+	}
+
+	/* Might be true during initialization. */
+	if (html->engine->painter == new_painter)
+		return;
+
+	html_gdk_painter_unrealize (HTML_GDK_PAINTER (old_painter));
+	if (html->engine->window != NULL)
+		html_gdk_painter_realize (
+			HTML_GDK_PAINTER (new_painter),
+			html->engine->window);
+
+	html_font_manager_set_default (
+		&new_painter->font_manager,
+		old_painter->font_manager.variable.face,
+		old_painter->font_manager.fixed.face,
+		old_painter->font_manager.var_size,
+		old_painter->font_manager.var_points,
+		old_painter->font_manager.fix_size,
+		old_painter->font_manager.fix_points);
+
+	html_engine_set_painter (html->engine, new_painter);
+	html_engine_schedule_redraw (html->engine);
+}
+
 static void
 action_indent_cb (GtkAction *action,
                   GtkhtmlEditor *editor)
@@ -564,67 +687,13 @@ action_indent_cb (GtkAction *action,
 	gtk_html_command (gtkhtml_editor_get_html (editor), "indent-more");
 }
 
-static void
-action_insert_face_cb (GtkhtmlFaceAction *action,
-                       GtkhtmlEditor *editor)
-{
-	GtkHTML *html;
-	HTMLObject *image;
-	GtkIconInfo *icon_info;
-	GtkIconTheme *icon_theme;
-	GtkhtmlFaceChooser *chooser;
-	GtkhtmlFace *face;
-	const gchar *filename;
-	gchar *uri = NULL;
-
-	html = gtkhtml_editor_get_html (editor);
-
-	chooser = GTKHTML_FACE_CHOOSER (action);
-	face = gtkhtml_face_chooser_get_current_face (chooser);
-	g_return_if_fail (face != NULL);
-
-	icon_theme = gtk_icon_theme_get_default ();
-	icon_info = gtk_icon_theme_lookup_icon (
-		icon_theme, face->icon_name, 16, 0);
-	g_return_if_fail (icon_info != NULL);
-
-	filename = gtk_icon_info_get_filename (icon_info);
-	if (filename != NULL)
-		uri = g_filename_to_uri (filename, NULL, NULL);
-	gtk_icon_info_free (icon_info);
-	g_return_if_fail (uri != NULL);
-
-	image = html_image_new (
-		html_engine_get_image_factory (html->engine),
-		uri, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
-		HTML_VALIGN_MIDDLE, FALSE);
-	html_image_set_alt (HTML_IMAGE (image), face->text_face);
-	html_engine_paste_object (
-		html->engine, image, html_object_get_length (image));
-
-	g_free (uri);
-}
-
 static void
 action_insert_html_file_cb (GtkToggleAction *action,
                             GtkhtmlEditor *editor)
 {
-	GFile *file;
-
-	file = gtkhtml_editor_run_open_dialog (
-		editor, _("Insert HTML File"), NULL, NULL);
-
-	/* User cancelled? */
-	if (file == NULL)
-		return;
-
-	/* XXX Need a way to cancel this. */
-	g_file_load_contents_async (
-		file, NULL, (GAsyncReadyCallback)
-		insert_html_file_ready_cb,
-		g_object_ref (editor));
-
-	g_object_unref (file);
+	gtkhtml_editor_insert_file (
+		editor, _("Insert HTML File"),
+		G_CALLBACK (insert_html_file_response_cb));
 }
 
 static void
@@ -658,6 +727,125 @@ action_insert_rule_cb (GtkAction *action,
 	gtk_action_activate (ACTION (PROPERTIES_RULE));
 }
 
+static void
+action_insert_face_angel_cb (GtkAction *action,
+                             GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, "O:-)");
+}
+
+static void
+action_insert_face_cool_cb (GtkAction *action,
+                            GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, "B-)");
+}
+
+static void
+action_insert_face_crying_cb (GtkAction *action,
+                              GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":'(");
+}
+
+static void
+action_insert_face_devilish_cb (GtkAction *action,
+                                GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ">:-)");
+}
+
+static void
+action_insert_face_embarrassed_cb (GtkAction *action,
+                                   GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":\"-)");
+}
+
+static void
+action_insert_face_kiss_cb (GtkAction *action,
+                            GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-*");
+}
+
+static void
+action_insert_face_monkey_cb (GtkAction *action,
+                              GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-(|)");
+}
+
+static void
+action_insert_face_plain_cb (GtkAction *action,
+                             GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-|");
+}
+
+static void
+action_insert_face_raspberry_cb (GtkAction *action,
+                                 GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-P");
+}
+
+static void
+action_insert_face_sad_cb (GtkAction *action,
+                           GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-(");
+}
+
+static void
+action_insert_face_smile_cb (GtkAction *action,
+                             GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-)");
+}
+
+static void
+action_insert_face_smile_big_cb (GtkAction *action,
+                                 GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-D");
+}
+
+static void
+action_insert_face_smirk_cb (GtkAction *action,
+                             GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-!");
+}
+
+static void
+action_insert_face_surprise_cb (GtkAction *action,
+                                GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-O");
+}
+
+static void
+action_insert_face_wink_cb (GtkAction *action,
+                            GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ";-)");
+}
+
+static void
+action_insert_smiley_9_cb (GtkAction *action,
+                           GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-/");
+}
+
+static void
+action_insert_smiley_26_cb (GtkAction *action,
+                            GtkhtmlEditor *editor)
+{
+	insert_emoticon (editor, action, ":-Q");
+}
+
 static void
 action_insert_table_cb (GtkAction *action,
                         GtkhtmlEditor *editor)
@@ -678,22 +866,9 @@ static void
 action_insert_text_file_cb (GtkAction *action,
                             GtkhtmlEditor *editor)
 {
-	GFile *file;
-
-	file = gtkhtml_editor_run_open_dialog (
-		editor, _("Insert Text File"), NULL, NULL);
-
-	/* User cancelled? */
-	if (file == NULL)
-		return;
-
-	/* XXX Need a way to cancel this. */
-	g_file_load_contents_async (
-		file, NULL, (GAsyncReadyCallback)
-		insert_text_file_ready_cb,
-		g_object_ref (editor));
-
-	g_object_unref (file);
+	gtkhtml_editor_insert_file (
+		editor, _("Insert Text File"),
+		G_CALLBACK (insert_text_file_response_cb));
 }
 
 static void
@@ -702,9 +877,6 @@ action_italic_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	if (gtk_toggle_action_get_active (action))
 		command = "italic-on";
 	else
@@ -746,7 +918,6 @@ action_language_cb (GtkToggleAction *action,
 	GtkhtmlSpellChecker *checker;
 	const gchar *language_code;
 	GtkAction *add_action;
-	GtkHTML *html;
 	GList *list;
 	guint length;
 	gchar *action_name;
@@ -788,82 +959,6 @@ action_language_cb (GtkToggleAction *action,
 	gtk_action_set_visible (ACTION (CONTEXT_SPELL_IGNORE), length > 0);
 
 	gtk_action_set_sensitive (ACTION (SPELL_CHECK), length > 0);
-
-	html = gtkhtml_editor_get_html (editor);
-	html_engine_spell_check (html->engine);
-}
-
-static void
-action_mode_cb (GtkRadioAction *action,
-                GtkRadioAction *current,
-                GtkhtmlEditor *editor)
-{
-	GtkActionGroup *action_group;
-	HTMLPainter *new_painter;
-	HTMLPainter *old_painter;
-	GtkHTML *html;
-	EditorMode mode;
-	gboolean html_mode;
-
-	html = gtkhtml_editor_get_html (editor);
-	mode = gtk_radio_action_get_current_value (current);
-	html_mode = (mode == EDITOR_MODE_HTML);
-
-	action_group = editor->priv->html_actions;
-	gtk_action_group_set_sensitive (action_group, html_mode);
-
-	action_group = editor->priv->html_context_actions;
-	gtk_action_group_set_visible (action_group, html_mode);
-
-        gtk_widget_set_sensitive (editor->priv->color_combo_box, html_mode);
-
-	if (html_mode)
-		gtk_widget_show (editor->priv->html_toolbar);
-	else
-		gtk_widget_hide (editor->priv->html_toolbar);
-
-	/* Certain paragraph styles are HTML-only. */
-	gtk_action_set_sensitive (ACTION (STYLE_H1), html_mode);
-	gtk_action_set_sensitive (ACTION (STYLE_H2), html_mode);
-	gtk_action_set_sensitive (ACTION (STYLE_H3), html_mode);
-	gtk_action_set_sensitive (ACTION (STYLE_H4), html_mode);
-	gtk_action_set_sensitive (ACTION (STYLE_H5), html_mode);
-	gtk_action_set_sensitive (ACTION (STYLE_H6), html_mode);
-	gtk_action_set_sensitive (ACTION (STYLE_ADDRESS), html_mode);
-
-	/* Swap painters. */
-
-	if (html_mode) {
-		new_painter = editor->priv->html_painter;
-		old_painter = editor->priv->plain_painter;
-	} else {
-		new_painter = editor->priv->plain_painter;
-		old_painter = editor->priv->html_painter;
-	}
-
-	/* Might be true during initialization. */
-	if (html->engine->painter == new_painter)
-		return;
-
-	html_gdk_painter_unrealize (HTML_GDK_PAINTER (old_painter));
-	if (html->engine->window != NULL)
-		html_gdk_painter_realize (
-			HTML_GDK_PAINTER (new_painter),
-			html->engine->window);
-
-	html_font_manager_set_default (
-		&new_painter->font_manager,
-		old_painter->font_manager.variable.face,
-		old_painter->font_manager.fixed.face,
-		old_painter->font_manager.var_size,
-		old_painter->font_manager.var_points,
-		old_painter->font_manager.fix_size,
-		old_painter->font_manager.fix_points);
-
-	html_engine_set_painter (html->engine, new_painter);
-	html_engine_schedule_redraw (html->engine);
-
-	g_object_notify (G_OBJECT (editor), "html-mode");
 }
 
 static void
@@ -874,9 +969,6 @@ action_monospaced_cb (GtkToggleAction *action,
 	GtkHTMLFontStyle and_mask;
 	GtkHTMLFontStyle or_mask;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	if (gtk_toggle_action_get_active (action)) {
 		and_mask = GTK_HTML_FONT_STYLE_MAX;
 		or_mask = GTK_HTML_FONT_STYLE_FIXED;
@@ -973,9 +1065,6 @@ action_style_cb (GtkRadioAction *action,
 {
 	const gchar *command = NULL;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	switch (gtk_radio_action_get_current_value (current)) {
 		case GTK_HTML_PARAGRAPH_STYLE_NORMAL:
 			command = "style-normal";
@@ -1062,9 +1151,6 @@ action_size_cb (GtkRadioAction *action,
 {
 	const gchar *command = NULL;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	switch (gtk_radio_action_get_current_value (current)) {
 		case GTK_HTML_FONT_STYLE_SIZE_1:
 			command = "size-minus-2";
@@ -1105,9 +1191,6 @@ action_strikethrough_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	if (gtk_toggle_action_get_active (action))
 		command = "strikeout-on";
 	else
@@ -1120,17 +1203,13 @@ static void
 action_test_url_cb (GtkAction *action,
                     GtkhtmlEditor *editor)
 {
-	GtkEntry *entry;
-	GtkWindow *window;
 	const gchar *text;
 
-	entry = GTK_ENTRY (WIDGET (LINK_PROPERTIES_URL_ENTRY));
-	window = GTK_WINDOW (WIDGET (LINK_PROPERTIES_WINDOW));
-
-	text = gtk_entry_get_text (entry);
+	text = gtk_entry_get_text (
+		GTK_ENTRY (WIDGET (LINK_PROPERTIES_URL_ENTRY)));
 
 	if (text != NULL && *text != '\0')
-		gtkhtml_editor_show_uri (window, text);
+		gnome_url_show (text, NULL);
 }
 
 static void
@@ -1139,9 +1218,6 @@ action_underline_cb (GtkToggleAction *action,
 {
 	const gchar *command;
 
-	if (editor->priv->ignore_style_change)
-		return;
-
 	if (gtk_toggle_action_get_active (action))
 		command = "underline-on";
 	else
@@ -1245,8 +1321,8 @@ static GtkActionEntry core_entries[] = {
 	{ "indent",
 	  GTK_STOCK_INDENT,
 	  N_("_Increase Indent"),
-	  "<Control>bracketright",
-	  N_("Increase Indent"),
+	  "<Control>bracketleft",
+	  NULL,
 	  G_CALLBACK (action_indent_cb) },
 
 	{ "insert-html-file",
@@ -1256,6 +1332,125 @@ static GtkActionEntry core_entries[] = {
 	  NULL,
 	  G_CALLBACK (action_insert_html_file_cb) },
 
+	{ "insert-face-angel",
+	  "face-angel",
+	  N_("_Angel"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_angel_cb) },
+
+	{ "insert-face-cool",
+	  "face-cool",
+	  N_("_Cool"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_cool_cb) },
+
+	{ "insert-face-crying",
+	  "face-crying",
+	  N_("Cr_ying"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_crying_cb) },
+
+	{ "insert-face-devilish",
+	  "face-devilish",
+	  N_("_Devilish"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_devilish_cb) },
+
+	{ "insert-face-embarrassed",
+	  "face-embarrassed",
+	  N_("_Embarrassed"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_embarrassed_cb) },
+
+	{ "insert-face-kiss",
+	  "face-kiss",
+	  N_("_Kiss"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_kiss_cb) },
+
+	{ "insert-face-monkey",
+	  "face-monkey",
+	  N_("_Monkey"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_monkey_cb) },
+
+	{ "insert-face-plain",
+	  "face-plain",
+	  N_("_Indifferent"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_plain_cb) },
+
+	{ "insert-face-raspberry",
+	  "face-raspberry",
+	  N_("_Tongue"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_raspberry_cb) },
+
+	{ "insert-face-sad",
+	  "face-sad",
+	  N_("_Frown"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_sad_cb) },
+
+	{ "insert-face-smile",
+	  "face-smile",
+	  N_("_Smile"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_smile_cb) },
+
+	{ "insert-face-smile-big",
+	  "face-smile-big",
+	  N_("_Laughing"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_smile_big_cb) },
+
+	{ "insert-face-smirk",
+	  "face-smirk",
+	  N_("Smi_rk"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_smirk_cb) },
+
+	{ "insert-face-surprise",
+	  "face-surprise",
+	  N_("Sur_prised"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_surprise_cb) },
+
+	{ "insert-face-wink",
+	  "face-wink",
+	  N_("_Wink"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_face_wink_cb) },
+
+	{ "insert-smiley-9",
+	  "stock_smiley-9",
+	  N_("_Undecided"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_smiley_9_cb) },
+
+	{ "insert-smiley-26",
+	  "stock_smiley-26",
+	  N_("S_ick"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_insert_smiley_26_cb) },
+
 	{ "insert-text-file",
 	  NULL,
 	  N_("Te_xt File..."),
@@ -1329,14 +1524,14 @@ static GtkActionEntry core_entries[] = {
 	{ "unindent",
 	  GTK_STOCK_UNINDENT,
 	  N_("_Decrease Indent"),
-	  "<Control>bracketleft",
-	  N_("Decrease Indent"),
+	  "<Control>bracketright",
+	  NULL,
 	  G_CALLBACK (action_unindent_cb) },
 
 	{ "wrap-lines",
 	  NULL,
 	  N_("_Wrap Lines"),
-	  "<Control>k",
+	  "<Control>l",
 	  NULL,
 	  G_CALLBACK (action_wrap_lines_cb) },
 
@@ -1370,6 +1565,13 @@ static GtkActionEntry core_entries[] = {
 	  NULL,
 	  NULL },
 
+	{ "insert-face-menu",
+	  NULL,
+	  N_("_Emoticon"),
+	  NULL,
+	  NULL,
+	  NULL },
+
 	{ "insert-menu",
 	  NULL,
 	  N_("_Insert"),
@@ -1399,47 +1601,41 @@ static GtkActionEntry core_entries[] = {
 	  NULL }
 };
 
+static GtkToggleActionEntry core_toggle_entries[] = {
+
+	{ "html-mode",
+	  NULL,
+	  N_("_HTML Mode"),
+	  NULL,
+	  NULL,
+	  G_CALLBACK (action_html_mode_cb),
+	  TRUE },
+};
+
 static GtkRadioActionEntry core_justify_entries[] = {
 
 	{ "justify-center",
 	  GTK_STOCK_JUSTIFY_CENTER,
 	  N_("_Center"),
-	  "<Control>e",
-	  N_("Center Alignment"),
+	  NULL,
+	  NULL,
 	  GTK_HTML_PARAGRAPH_ALIGNMENT_CENTER },
 
 	{ "justify-left",
 	  GTK_STOCK_JUSTIFY_LEFT,
 	  N_("_Left"),
-	  "<Control>l",
-	  N_("Left Alignment"),
+	  NULL,
+	  NULL,
 	  GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT },
 
 	{ "justify-right",
 	  GTK_STOCK_JUSTIFY_RIGHT,
 	  N_("_Right"),
-	  "<Control>r",
-	  N_("Right Alignment"),
+	  NULL,
+	  NULL,
 	  GTK_HTML_PARAGRAPH_ALIGNMENT_RIGHT }
 };
 
-static GtkRadioActionEntry core_mode_entries[] = {
-
-	{ "mode-html",
-	  NULL,
-	  N_("_HTML"),
-	  NULL,
-	  N_("HTML editing mode"),
-	  EDITOR_MODE_HTML },
-
-	{ "mode-plain",
-	  NULL,
-	  N_("Plain _Text"),
-	  NULL,
-	  N_("Plain text editing mode"),
-	  EDITOR_MODE_TEXT }
-};
-
 static GtkRadioActionEntry core_style_entries[] = {
 
 	{ "style-normal",
@@ -1546,30 +1742,28 @@ static GtkActionEntry html_entries[] = {
 	  "insert-image",
 	  N_("_Image..."),
 	  NULL,
-	  N_("Insert Image"),
+	  NULL,
 	  G_CALLBACK (action_insert_image_cb) },
 
 	{ "insert-link",
 	  "insert-link",
 	  N_("_Link..."),
 	  NULL,
-	  N_("Insert Link"),
+	  NULL,
 	  G_CALLBACK (action_insert_link_cb) },
 
 	{ "insert-rule",
 	  "stock_insert-rule",
-	  /* Translators: 'Rule' here means a horizontal line in an HTML text */
 	  N_("_Rule..."),
 	  NULL,
-	  /* Translators: 'Rule' here means a horizontal line in an HTML text */
-	  N_("Insert Rule"),
+	  NULL,
 	  G_CALLBACK (action_insert_rule_cb) },
 
 	{ "insert-table",
 	  "stock_insert-table",
 	  N_("_Table..."),
 	  NULL,
-	  N_("Insert Table"),
+	  NULL,
 	  G_CALLBACK (action_insert_table_cb) },
 
 	{ "properties-cell",
@@ -1602,7 +1796,6 @@ static GtkActionEntry html_entries[] = {
 
 	{ "properties-rule",
 	  NULL,
-	  /* Translators: 'Rule' here means a horizontal line in an HTML text */
 	  N_("_Rule..."),
 	  NULL,
 	  NULL,
@@ -1638,7 +1831,7 @@ static GtkToggleActionEntry html_toggle_entries[] = {
 	  GTK_STOCK_BOLD,
 	  N_("_Bold"),
 	  "<Control>b",
-	  N_("Bold"),
+	  NULL,
 	  G_CALLBACK (action_bold_cb),
 	  FALSE },
 
@@ -1646,7 +1839,7 @@ static GtkToggleActionEntry html_toggle_entries[] = {
 	  GTK_STOCK_ITALIC,
 	  N_("_Italic"),
 	  "<Control>i",
-	  N_("Italic"),
+	  NULL,
 	  G_CALLBACK (action_italic_cb),
 	  FALSE },
 
@@ -1654,7 +1847,7 @@ static GtkToggleActionEntry html_toggle_entries[] = {
 	  "stock_text-monospaced",
 	  N_("_Plain Text"),
 	  "<Control>t",
-	  N_("Plain Text"),
+	  NULL,
 	  G_CALLBACK (action_monospaced_cb),
 	  FALSE },
 
@@ -1662,7 +1855,7 @@ static GtkToggleActionEntry html_toggle_entries[] = {
 	  GTK_STOCK_STRIKETHROUGH,
 	  N_("_Strikethrough"),
 	  NULL,
-	  N_("Strikethrough"),
+	  NULL,
 	  G_CALLBACK (action_strikethrough_cb),
 	  FALSE },
 
@@ -1670,7 +1863,7 @@ static GtkToggleActionEntry html_toggle_entries[] = {
 	  GTK_STOCK_UNDERLINE,
 	  N_("_Underline"),
 	  "<Control>u",
-	  N_("Underline"),
+	  NULL,
 	  G_CALLBACK (action_underline_cb),
 	  FALSE }
 };
@@ -1679,7 +1872,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-minus-two",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("-2"),
 	  NULL,
 	  NULL,
@@ -1687,7 +1879,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-minus-one",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("-1"),
 	  NULL,
 	  NULL,
@@ -1695,7 +1886,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-plus-zero",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("+0"),
 	  NULL,
 	  NULL,
@@ -1703,7 +1893,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-plus-one",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("+1"),
 	  NULL,
 	  NULL,
@@ -1711,7 +1900,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-plus-two",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("+2"),
 	  NULL,
 	  NULL,
@@ -1719,7 +1907,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-plus-three",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("+3"),
 	  NULL,
 	  NULL,
@@ -1727,7 +1914,6 @@ static GtkRadioActionEntry html_size_entries[] = {
 
 	{ "size-plus-four",
 	  NULL,
-	  /* Translators: This is a font size level. It is shown on a tool bar. Please keep it as short as possible. */
 	  N_("+4"),
 	  NULL,
 	  NULL,
@@ -1777,7 +1963,6 @@ static GtkActionEntry context_entries[] = {
 
 	{ "context-delete-table-menu",
 	  NULL,
-	  /* Translators: Popup menu item caption, containing all the Delete options for a table */
 	  N_("Table Delete"),
 	  NULL,
 	  NULL,
@@ -1792,7 +1977,6 @@ static GtkActionEntry context_entries[] = {
 
 	{ "context-insert-table-menu",
 	  NULL,
-	  /* Translators: Popup menu item caption, containing all the Insert options for a table */
 	  N_("Table Insert"),
 	  NULL,
 	  NULL,
@@ -1895,7 +2079,6 @@ static GtkActionEntry html_context_entries[] = {
 
 	{ "context-properties-rule",
 	  NULL,
-	  /* Translators: 'Rule' here means a horizontal line in an HTML text */
 	  N_("Rule..."),
 	  NULL,
 	  NULL,
@@ -1951,15 +2134,6 @@ static GtkActionEntry spell_context_entries[] = {
 	  N_("Add Word To"),
 	  NULL,
 	  NULL,
-	  NULL },
-
-	/* Menus */
-
-	{ "context-more-suggestions-menu",
-	  NULL,
-	  N_("More Suggestions"),
-	  NULL,
-	  NULL,
 	  NULL }
 };
 
@@ -2055,10 +2229,8 @@ editor_actions_setup_spell_check_menu (GtkhtmlEditor *editor)
 
 		/* Add an item to the "Add Word To" menu. */
 
+		action_label = g_strdup_printf ("%s Dictionary", name);
 		action_name = g_strdup_printf ("context-spell-add-%s", code);
-		/* Translators: %s will be replaced with the actual dictionary name,
-		   where a user can add a word to. This is part of an "Add Word To" submenu. */
-		action_label = g_strdup_printf (_("%s Dictionary"), name);
 
 		action = gtk_action_new (
 			action_name, action_label, NULL, NULL);
@@ -2091,32 +2263,28 @@ editor_actions_setup_spell_check_menu (GtkhtmlEditor *editor)
 void
 gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 {
-	GtkAction *action;
 	GtkActionGroup *action_group;
 	GtkUIManager *manager;
-	const gchar *domain;
 
 	g_return_if_fail (GTKHTML_IS_EDITOR (editor));
 
 	manager = gtkhtml_editor_get_ui_manager (editor);
-	domain = GETTEXT_PACKAGE;
 
 	/* Core Actions */
 	action_group = editor->priv->core_actions;
-	gtk_action_group_set_translation_domain (action_group, domain);
+	gtk_action_group_set_translation_domain (
+		action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (
 		action_group, core_entries,
 		G_N_ELEMENTS (core_entries), editor);
+	gtk_action_group_add_toggle_actions (
+		action_group, core_toggle_entries,
+		G_N_ELEMENTS (core_toggle_entries), editor);
 	gtk_action_group_add_radio_actions (
 		action_group, core_justify_entries,
 		G_N_ELEMENTS (core_justify_entries),
 		GTK_HTML_PARAGRAPH_ALIGNMENT_LEFT,
 		G_CALLBACK (action_justify_cb), editor);
-	gtk_action_group_add_radio_actions (
-		action_group, core_mode_entries,
-		G_N_ELEMENTS (core_mode_entries),
-		EDITOR_MODE_HTML,
-		G_CALLBACK (action_mode_cb), editor);
 	gtk_action_group_add_radio_actions (
 		action_group, core_style_entries,
 		G_N_ELEMENTS (core_style_entries),
@@ -2124,19 +2292,10 @@ gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 		G_CALLBACK (action_style_cb), editor);
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 
-	/* Face Action */
-	action = gtkhtml_face_action_new (
-		"insert-face", _("_Emoticon"),
-		_("Insert Emoticon"), NULL);
-	g_object_set (action, "icon-name", "face-smile", NULL);
-	g_signal_connect (
-		action, "item-activated",
-		G_CALLBACK (action_insert_face_cb), editor);
-	gtk_action_group_add_action (action_group, action);
-
 	/* Core Actions (HTML only) */
 	action_group = editor->priv->html_actions;
-	gtk_action_group_set_translation_domain (action_group, domain);
+	gtk_action_group_set_translation_domain (
+		action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (
 		action_group, html_entries,
 		G_N_ELEMENTS (html_entries), editor);
@@ -2152,7 +2311,8 @@ gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 
 	/* Context Menu Actions */
 	action_group = editor->priv->context_actions;
-	gtk_action_group_set_translation_domain (action_group, domain);
+	gtk_action_group_set_translation_domain (
+		action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (
 		action_group, context_entries,
 		G_N_ELEMENTS (context_entries), editor);
@@ -2160,7 +2320,8 @@ gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 
 	/* Context Menu Actions (HTML only) */
 	action_group = editor->priv->html_context_actions;
-	gtk_action_group_set_translation_domain (action_group, domain);
+	gtk_action_group_set_translation_domain (
+		action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (
 		action_group, html_context_entries,
 		G_N_ELEMENTS (html_context_entries), editor);
@@ -2168,7 +2329,8 @@ gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 
 	/* Context Menu Actions (spell check only) */
 	action_group = editor->priv->spell_check_actions;
-	gtk_action_group_set_translation_domain (action_group, domain);
+	gtk_action_group_set_translation_domain (
+		action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (
 		action_group, spell_context_entries,
 		G_N_ELEMENTS (spell_context_entries), editor);
@@ -2200,7 +2362,6 @@ gtkhtml_editor_actions_init (GtkhtmlEditor *editor)
 		"short-label", _("_Link"), NULL);
 	g_object_set (
 		G_OBJECT (ACTION (INSERT_RULE)),
-		/* Translators: 'Rule' here means a horizontal line in an HTML text */
 		"short-label", _("_Rule"), NULL);
 	g_object_set (
 		G_OBJECT (ACTION (INSERT_TABLE)),
