@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * color-palette.h - A color selector palette
  * Copyright 2000, 2001, Ximian, Inc.
  *
@@ -30,10 +30,7 @@
 #ifndef GI_COLOR_PALETTE_H
 #define GI_COLOR_PALETTE_H
 
-#include <gtk/gtkcolorbutton.h>
-#include <gtk/gtktooltips.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include "gi-color-group.h"
 
@@ -43,7 +40,6 @@ typedef struct _ColorNamePair ColorNamePair;
 
 typedef struct _ColorPalette {
 	GtkVBox          vbox;
-	GtkTooltips      *tool_tip;
 	GtkColorButton   *button;
 	/*
 	 * Array of colors
@@ -55,7 +51,7 @@ typedef struct _ColorPalette {
 	/* The current color */
 	GdkColor *current_color;
 	gboolean  current_is_default;
-	
+
         /*
 	 * Position of the last possible position
 	 * for custom colors in **items
@@ -72,14 +68,14 @@ typedef struct _ColorPalette {
 
 	/* The table with our default color names */
 	ColorNamePair *default_set;
-	
+
 	/* The color group to which we belong */
 	ColorGroup *color_group;
 } ColorPalette;
 
 typedef struct {
 	GtkVBoxClass parent_class;
-	
+
 	/* Signals emited by this widget */
 	void (* color_changed) (ColorPalette *color_palette, GdkColor *color,
 				gboolean custom, gboolean by_user, gboolean is_default);
