@@ -85,8 +85,8 @@ finalize (GObject *object)
 
 static void
 begin (HTMLPainter *painter,
-       gint x1, gint y1,
-       gint x2, gint y2)
+       int x1, int y1,
+       int x2, int y2)
 {
 	HTMLPrinter *printer;
 	GtkPrintContext *pc;
@@ -94,7 +94,7 @@ begin (HTMLPainter *painter,
 	gdouble printer_x2, printer_y2;
 	cairo_t *cr;
 #ifdef PRINTER_DEBUG
-	gdouble dash[2];
+	gdouble dash [2];
 #endif
 	printer = HTML_PRINTER (painter);
 	g_return_if_fail (printer);
@@ -113,8 +113,8 @@ begin (HTMLPainter *painter,
 	cairo_rectangle (cr, printer_x1, printer_y1, printer_x2, printer_y2);
 #ifdef PRINTER_DEBUG
 	cairo_save (cr);
-	dash[0] = 10.0;
-	dash[1] = 10.0;
+	dash [0] = 10.0;
+	dash [1] = 10.0;
 	cairo_set_source_rgb (cr, .5, .5, .5);
 	cairo_set_line_width (cr, .3);
 	cairo_set_dash (cr, 2, dash, .0);
@@ -222,8 +222,8 @@ draw_line (HTMLPainter *painter,
 	   gint x2, gint y2)
 {
 	HTMLPrinter *printer;
-	gdouble printer_x1, printer_y1;
-	gdouble printer_x2, printer_y2;
+	double printer_x1, printer_y1;
+	double printer_x2, printer_y2;
 	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
@@ -392,13 +392,15 @@ static void
 draw_pixmap (HTMLPainter *painter, GdkPixbuf *pixbuf, gint x, gint y, gint scale_width, gint scale_height, const GdkColor *color)
 {
 	HTMLPrinter *printer;
-	gdouble print_x, print_y;
-	gdouble print_scale_width, print_scale_height;
+	double print_x, print_y;
+	double print_scale_width, print_scale_height;
+	double page_height;
 	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
-	g_return_if_fail (printer->context != NULL);
+       	g_return_if_fail (printer->context != NULL);
 	cr = gtk_print_context_get_cairo_context (printer->context);
+	page_height =(double) gtk_print_context_get_height (printer->context);
 	print_x = SCALE_ENGINE_TO_GNOME_PRINT (x);
 	print_y = SCALE_ENGINE_TO_GNOME_PRINT (y);
 	print_scale_width  = SCALE_ENGINE_TO_GNOME_PRINT (scale_width);
@@ -414,8 +416,8 @@ static void
 fill_rect (HTMLPainter *painter, gint x, gint y, gint width, gint height)
 {
 	HTMLPrinter *printer;
-	gdouble printer_x, printer_y;
-	gdouble printer_width, printer_height;
+	double printer_x, printer_y;
+	double printer_width, printer_height;
 	cairo_t *cr;
 
 	printer = HTML_PRINTER (painter);
@@ -448,11 +450,11 @@ draw_lines (HTMLPrinter *printer, double x, double y, double width, PangoAnalysi
 	cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
 
 	if (properties->underline) {
-		gdouble thickness = pango_units_to_double (pango_font_metrics_get_underline_thickness (metrics));
-		gdouble position = pango_units_to_double (pango_font_metrics_get_underline_position (metrics));
-		gdouble ly = y + position - thickness / 2;
+		double thickness = pango_units_to_double (pango_font_metrics_get_underline_thickness (metrics));
+		double position = pango_units_to_double (pango_font_metrics_get_underline_position (metrics));
+		double ly = y + position - thickness / 2;
 
-		cairo_new_path (cr);
+	    	cairo_new_path (cr);
 		cairo_move_to (cr, x, ly + 4);
 		cairo_line_to (cr, x + width, ly + 4);
 		cairo_set_line_width (cr, thickness);
@@ -460,9 +462,9 @@ draw_lines (HTMLPrinter *printer, double x, double y, double width, PangoAnalysi
 	}
 
 	if (properties->strikethrough) {
-		gdouble thickness = pango_units_to_double (pango_font_metrics_get_strikethrough_thickness (metrics));
-		gdouble position = pango_units_to_double (pango_font_metrics_get_strikethrough_position (metrics));
-		gdouble ly = y + position - thickness / 2;
+		double thickness = pango_units_to_double (pango_font_metrics_get_strikethrough_thickness (metrics));
+		double position = pango_units_to_double (pango_font_metrics_get_strikethrough_position (metrics));
+		double ly = y + position - thickness / 2;
 
 		cairo_new_path (cr);
 		cairo_move_to (cr, x, ly - 8);
@@ -493,12 +495,12 @@ draw_glyphs (HTMLPainter *painter, gint x, gint y, PangoItem *item, PangoGlyphSt
 	html_pango_get_item_properties (item, &properties);
 	pango_glyph_string_extents (glyphs, item->analysis.font, NULL, &log_rect);
 
-	if (properties.bg_color) {
+       	if (properties.bg_color) {
 		   cairo_set_source_rgb (cr,
 					 properties.bg_color->red / 65535.0,
 					 properties.bg_color->green / 65535.0,
 					 properties.bg_color->blue / 65535.0);
-			cairo_rectangle (cr,
+		        cairo_rectangle (cr,
 					 print_x,
 					 print_y + pango_units_to_double (log_rect.y + log_rect.height),
 					 pango_units_to_double (log_rect.width),
@@ -512,7 +514,7 @@ draw_glyphs (HTMLPainter *painter, gint x, gint y, PangoItem *item, PangoGlyphSt
 					 properties.fg_color->green / 65535.0,
 					 properties.fg_color->blue / 65535.0);
 	} else {
-			cairo_set_source_rgb (cr, 0., 0., 0.);
+		        cairo_set_source_rgb (cr, 0., 0., 0.);
 	}
 	cairo_move_to (cr, print_x, print_y);
 	pango_cairo_show_glyph_string (cr,
@@ -652,7 +654,7 @@ html_printer_get_type (void)
 HTMLPainter *
 html_printer_new (GtkWidget *widget, GtkPrintContext *context)
 {
-	GtkStyle *style;
+	const PangoFontDescription *desc;
 	HTMLPrinter *printer;
 	HTMLPainter *painter;
 
@@ -661,12 +663,9 @@ html_printer_new (GtkWidget *widget, GtkPrintContext *context)
 
 	painter = HTML_PAINTER (printer);
 	html_painter_set_widget (painter, widget);
-	style = gtk_widget_get_style (widget);
-	painter->pango_context =
-		gtk_print_context_create_pango_context (context);
-	pango_context_set_font_description (
-		painter->pango_context, style->font_desc);
-
+	desc = pango_font_description_from_string ("sans 8");/* FIXME font hardcoded*/
+	painter->pango_context = gtk_print_context_create_pango_context (context);
+	pango_context_set_font_description (painter->pango_context, desc);
 	return painter;
 
 }
@@ -675,7 +674,7 @@ html_printer_new (GtkWidget *widget, GtkPrintContext *context)
 guint
 html_printer_get_page_width (HTMLPrinter *printer)
 {
-	gdouble printer_width;
+	double printer_width;
 	guint engine_width;
 
 	g_return_val_if_fail (printer != NULL, 0);
@@ -690,7 +689,7 @@ html_printer_get_page_width (HTMLPrinter *printer)
 guint
 html_printer_get_page_height (HTMLPrinter *printer)
 {
-	gdouble printer_height;
+	double printer_height;
 	guint engine_height;
 
 	g_return_val_if_fail (printer != NULL, 0);
