@@ -33,14 +33,13 @@
 enum {
 	HTML_TOKENIZER_BEGIN_SIGNAL,
 	HTML_TOKENIZER_END_SIGNAL,
-	HTML_TOKENIZER_CHANGECONTENT_SIGNAL,
-	HTML_TOKENIZER_CHANGEENGINE_SIGNAL,
 	HTML_TOKENIZER_LAST_SIGNAL
 };
 
 static guint html_tokenizer_signals[HTML_TOKENIZER_LAST_SIGNAL] = { 0 };
 
 #define TOKEN_BUFFER_SIZE (1 << 10)
+#define INVALID_CHARACTER_MARKER '?'
 
 #define dt(x)
 
@@ -53,7 +52,6 @@ struct _HTMLTokenBuffer {
 	gint used;
 	gchar * data;
 };
-
 struct _HTMLTokenizerPrivate {
 
 	/* token buffers list */
@@ -89,7 +87,9 @@ struct _HTMLTokenizerPrivate {
 	gboolean textarea; /* Are we in a <textarea> block? */
 	gint     pre; /* Are we in a <pre> block? */
 	gboolean select; /* Are we in a <select> block? */
+	gboolean charEntity; /* Are we in an &... sequence? */
 	gboolean extension; /* Are we in an <!-- +GtkHTML: sequence? */
+	gboolean aTag; /* Are we in a <a/> tag*/
 
 	enum {
 		NoneDiscard = 0,
@@ -104,6 +104,7 @@ struct _HTMLTokenizerPrivate {
 		TabPending
 	} pending;
 
+
 	gchar searchBuffer[20];
 	gint searchCount;
 	gint searchGtkHTMLCount;
@@ -116,13 +117,9 @@ struct _HTMLTokenizerPrivate {
 	GList *blocking; /* Blocking tokens */
 
 	const gchar *searchFor;
-
-	gboolean enableconvert;
-
-	gchar * content_type;
-	/*convert*/
-	GIConv iconv_cd;
-
+	gboolean utf8;
+	gchar utf8_buffer[7];
+	gint utf8_length;
 };
 
 static const gchar *commentStart = "<!--";
@@ -147,34 +144,24 @@ static void           html_tokenizer_append_token_buffer (HTMLTokenizer *t,
 
 /* default implementations of tokenization functions */
 static void     html_tokenizer_finalize             (GObject *);
-static void     html_tokenizer_real_change          (HTMLTokenizer *, const gchar *content_type);
-static void     html_tokenizer_real_begin           (HTMLTokenizer *, const gchar *content_type);
-static void     html_tokenizer_real_engine_type (HTMLTokenizer *t, gboolean engine_type);
-static void     html_tokenizer_real_write           (HTMLTokenizer *, const gchar *str, gsize size);
+static void     html_tokenizer_real_begin           (HTMLTokenizer *, gchar *content_type);
+static void     html_tokenizer_real_write           (HTMLTokenizer *, const gchar *str, size_t size);
 static void     html_tokenizer_real_end             (HTMLTokenizer *);
-static const gchar *
-				html_tokenizer_real_get_content_type(HTMLTokenizer *);
-static gboolean
-				html_tokenizer_real_get_engine_type(HTMLTokenizer *);
 static gchar   *html_tokenizer_real_peek_token      (HTMLTokenizer *);
 static gchar   *html_tokenizer_real_next_token      (HTMLTokenizer *);
 static gboolean html_tokenizer_real_has_more_tokens (HTMLTokenizer *);
-static gchar   *html_tokenizer_converted_token (HTMLTokenizer *t,const gchar * token);
 
 static HTMLTokenizer *html_tokenizer_real_clone     (HTMLTokenizer *);
 
 /* blocking tokens */
-static const gchar       *html_tokenizer_blocking_get_name   (HTMLTokenizer  *t);
+static gchar             *html_tokenizer_blocking_get_name   (HTMLTokenizer  *t);
 static void               html_tokenizer_blocking_pop        (HTMLTokenizer  *t);
 static void               html_tokenizer_blocking_push       (HTMLTokenizer  *t,
 							      HTMLTokenType   tt);
 static void               html_tokenizer_tokenize_one_char   (HTMLTokenizer  *t,
 							      const gchar  **src);
-static void				  add_char(HTMLTokenizer *t, gchar c);
 
-gboolean				  is_need_convert(const gchar * token);
-
-gchar *					  html_tokenizer_convert_entity(gchar * token);
+static void               add_unichar(HTMLTokenizer *t, gunichar wc);
 
 static GObjectClass *parent_class = NULL;
 
@@ -185,26 +172,6 @@ html_tokenizer_class_init (HTMLTokenizerClass *klass)
 
 	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 
-	html_tokenizer_signals[HTML_TOKENIZER_CHANGECONTENT_SIGNAL] =
-		g_signal_new ("change",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (HTMLTokenizerClass, change),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
-			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
-
-	html_tokenizer_signals[HTML_TOKENIZER_CHANGEENGINE_SIGNAL] =
-		g_signal_new ("engine",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (HTMLTokenizerClass, engine),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__POINTER,
-			      G_TYPE_NONE,
-			      1, G_TYPE_POINTER);
-
 	html_tokenizer_signals[HTML_TOKENIZER_BEGIN_SIGNAL] =
 		g_signal_new ("begin",
 			      G_TYPE_FROM_CLASS (klass),
@@ -215,7 +182,7 @@ html_tokenizer_class_init (HTMLTokenizerClass *klass)
 			      G_TYPE_NONE,
 			      1, G_TYPE_POINTER);
 
-	html_tokenizer_signals[HTML_TOKENIZER_END_SIGNAL] =
+	html_tokenizer_signals [HTML_TOKENIZER_END_SIGNAL] =
 		g_signal_new ("end",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -227,16 +194,12 @@ html_tokenizer_class_init (HTMLTokenizerClass *klass)
 
 	object_class->finalize = html_tokenizer_finalize;
 
-	klass->change     = html_tokenizer_real_change;
-	klass->engine     = html_tokenizer_real_engine_type;
 	klass->begin      = html_tokenizer_real_begin;
 	klass->end        = html_tokenizer_real_end;
 
 	klass->write      = html_tokenizer_real_write;
 	klass->peek_token = html_tokenizer_real_peek_token;
 	klass->next_token = html_tokenizer_real_next_token;
-	klass->get_content_type = html_tokenizer_real_get_content_type;
-	klass->get_engine_type = html_tokenizer_real_get_engine_type;
 	klass->has_more   = html_tokenizer_real_has_more_tokens;
 	klass->clone      = html_tokenizer_real_clone;
 }
@@ -269,7 +232,9 @@ html_tokenizer_init (HTMLTokenizer *t)
 	p->textarea = FALSE;
 	p->pre = 0;
 	p->select = FALSE;
+	p->charEntity = FALSE;
 	p->extension = FALSE;
+	p->aTag = FALSE;
 
 	p->discard = NoneDiscard;
 	p->pending = NonePending;
@@ -285,11 +250,6 @@ html_tokenizer_init (HTMLTokenizer *t)
 	p->blocking = NULL;
 
 	p->searchFor = NULL;
-
-	/* Use old logic and not convert charset */
-	p->enableconvert = FALSE;
-
-	p->content_type = g_strdup ("html/text; charset=utf-8");
 }
 
 static void
@@ -299,22 +259,16 @@ html_tokenizer_finalize (GObject *obj)
 
 	html_tokenizer_reset (t);
 
-	if (is_valid_g_iconv (t->priv->iconv_cd))
-		g_iconv_close (t->priv->iconv_cd);
-
-	if (t->priv->content_type)
-		g_free(t->priv->content_type);
-
 	g_free (t->priv);
 	t->priv = NULL;
 
-    G_OBJECT_CLASS (parent_class)->finalize (obj);
+        G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
-GType
+GtkType
 html_tokenizer_get_type (void)
 {
-	static GType html_tokenizer_type = 0;
+	static GtkType html_tokenizer_type = 0;
 
 	if (!html_tokenizer_type) {
 		static const GTypeInfo html_tokenizer_info = {
@@ -365,8 +319,8 @@ html_token_buffer_append_token (HTMLTokenBuffer * buf, const gchar *token, gint
 	/* copy token and terminate with zero */
 	strncpy (buf->data + buf->used, token, len);
 	buf->used += len;
-	buf->data[buf->used] = 0;
-	buf->used++;
+	buf->data [buf->used] = 0;
+	buf->used ++;
 
 	dt(printf ("html_token_buffer_append_token: '%s'\n", buf->data + buf->used - 1 - len));
 
@@ -416,164 +370,7 @@ html_tokenizer_real_peek_token (HTMLTokenizer *t)
 		token = buffer->data;
 	}
 
-	return html_tokenizer_converted_token (t,token);
-}
-
-/* test iconv for valid*/
-gboolean
-is_valid_g_iconv (const GIConv iconv_cd)
-{
-	return iconv_cd != NULL && iconv_cd != (GIConv)-1;
-}
-
-/*Convert only chars when code >127*/
-gboolean
-is_need_convert (const gchar * token)
-{
-	gint i=strlen (token);
-	for (;i>=0;i--)
-		if (token[i]&128)
-			return TRUE;
-	return FALSE;
-}
-
-/*Convert entity values in already converted to right charset token*/
-gchar *
-html_tokenizer_convert_entity (gchar *token)
-{
-	gchar *full_pos;
-	gchar *resulted;
-	gchar *write_pos;
-	gchar *read_pos;
-
-	if (token == NULL)
-		return NULL;
-
-	/*stop pointer*/
-	full_pos = token + strlen (token);
-	resulted = g_new (gchar, strlen (token) + 1);
-	write_pos = resulted;
-	read_pos = token;
-	while (read_pos < full_pos) {
-		gsize count_chars = strcspn (read_pos, "&");
-		memcpy (write_pos, read_pos, count_chars);
-		write_pos += count_chars;
-		read_pos += count_chars;
-		/*may be end string?*/
-		if (read_pos < full_pos)
-			if (*read_pos == '&') {
-				/*value to add*/
-				gunichar value = INVALID_ENTITY_CHARACTER_MARKER;
-				/*skip not needed &*/
-				read_pos++;
-				count_chars = strcspn (read_pos, ";");
-				if (count_chars < 14 && count_chars > 1) {
-					/*save for recovery*/
-					gchar save_gchar = *(read_pos + count_chars);
-					*(read_pos + count_chars)=0;
-					/* &#******; */
-					if (*read_pos == '#') {
-						/* &#1234567 */
-						if (isdigit (*(read_pos + 1))) {
-							value=strtoull (read_pos + 1, NULL, 10);
-						/* &#xdd; */
-						} else if (*(read_pos + 1) == 'x') {
-							value=strtoull (read_pos + 2, NULL, 16);
-						}
-					} else {
-						value = html_entity_parse (read_pos, strlen (read_pos));
-					}
-					if (*read_pos == '#' || value != INVALID_ENTITY_CHARACTER_MARKER) {
-						write_pos += g_unichar_to_utf8 (value, write_pos);
-						read_pos += (count_chars + 1);
-					} else {
-						/*recovery old value - it's not entity*/
-						write_pos += g_unichar_to_utf8 ('&', write_pos);
-						*(read_pos + count_chars) = save_gchar;
-					}
-				}
-				else
-					/*very large string*/
-					write_pos += g_unichar_to_utf8 ('&', write_pos);
-			}
-	}
-	*write_pos = 0;
-	free (token);
-
-	return resulted;
-}
-
-/*convert text to utf8 - allways alloc memmory*/
-gchar *
-convert_text_encoding (const GIConv iconv_cd,
-                       const gchar *token)
-{
-	gsize currlength;
-	gchar *newbuffer;
-	gchar *returnbuffer;
-	const gchar *current;
-	gsize newlength;
-	gsize oldlength;
-
-	if (token == NULL)
-		return NULL;
-
-	if (is_valid_g_iconv (iconv_cd) && is_need_convert (token)) {
-		currlength = strlen (token);
-		current = token;
-		newlength = currlength*7+1;
-		oldlength = newlength;
-		newbuffer = g_new (gchar, newlength);
-		returnbuffer = newbuffer;
-
-		while (currlength > 0) {
-			/*function not change current, but g_iconv use not const source*/
-			g_iconv (iconv_cd, (gchar **)&current, &currlength, &newbuffer, &newlength);
-			if (currlength > 0) {
-				g_warning ("IconvError=%s", current);
-				*newbuffer = INVALID_ENTITY_CHARACTER_MARKER;
-				newbuffer++;
-				current++;
-				currlength--;
-				newlength--;
-			}
-		}
-		returnbuffer[oldlength - newlength] = '\0';
-		returnbuffer = g_realloc (returnbuffer, oldlength - newlength + 1);
-		return returnbuffer;
-	}
-	return g_strdup (token);
-}
-
-static gchar *
-html_tokenizer_converted_token (HTMLTokenizer *t,
-                                const gchar * token)
-{
-	if (token != NULL) {
-		struct _HTMLTokenizerPrivate *p = t->priv;
-		return html_tokenizer_convert_entity (convert_text_encoding (p->iconv_cd, token));
-	}
-
-	return NULL;
-}
-
-static const gchar *
-html_tokenizer_real_get_content_type (HTMLTokenizer *t)
-{
-	struct _HTMLTokenizerPrivate *p = t->priv;
-
-	if (p->content_type)
-		return p->content_type;
-
-	return NULL;
-}
-
-static gboolean
-html_tokenizer_real_get_engine_type (HTMLTokenizer *t)
-{
-	struct _HTMLTokenizerPrivate *p = t->priv;
-
-	return p->enableconvert;
+	return token;
 }
 
 static gchar *
@@ -615,7 +412,7 @@ html_tokenizer_real_next_token (HTMLTokenizer *t)
 	p->tokens_num--;
 	g_assert (p->tokens_num >= 0);
 
-	return html_tokenizer_converted_token (t, token);
+	return token;
 }
 
 static gboolean
@@ -663,102 +460,14 @@ html_tokenizer_reset (HTMLTokenizer *t)
 	p->scriptCode = NULL;
 }
 
-static gboolean
-charset_is_utf8 (const gchar *content_type)
+static gint
+charset_is_utf8 (gchar *content_type)
 {
-	return content_type && strstr (content_type, "=utf-8") != NULL;
-}
-
-static gboolean
-is_text (const gchar *content_type)
-{
-	return content_type && strstr (content_type, "text/") != NULL;
-}
-
-static const gchar *
-get_encoding_from_content_type(const gchar * content_type)
-{
-	gchar * charset;
-	if (content_type)
-	{
-		charset =  g_strrstr (content_type, "charset=");
-		if (charset != NULL)
-			return charset + strlen ("charset=");
-		charset =  g_strrstr (content_type, "encoding=");
-		if (charset != NULL)
-			return charset + strlen ("encoding=");
-
-	}
-	return NULL;
-}
-
-GIConv
-generate_iconv_from(const gchar * content_type)
-{
-	if (content_type)
-		if (!charset_is_utf8(content_type))
-		{
-			const gchar * encoding = get_encoding_from_content_type (content_type);
-			if (encoding)
-				return g_iconv_open ("utf-8", encoding);
-		}
-	return NULL;
-}
-
-GIConv
-generate_iconv_to(const gchar * content_type)
-{
-	if (content_type)
-		if (!charset_is_utf8 (content_type))
-		{
-			const gchar * encoding = get_encoding_from_content_type (content_type);
-			if (encoding)
-				return g_iconv_open (encoding, "utf-8");
-		}
-	return NULL;
+	return content_type && strstr (content_type, "charset=utf-8") != NULL;
 }
 
 static void
-html_tokenizer_real_engine_type (HTMLTokenizer *t, gboolean engine_type)
-{
-	struct _HTMLTokenizerPrivate *p;
-	p = t->priv;
-
-	p->enableconvert = engine_type;
-}
-
-static void
-html_tokenizer_real_change (HTMLTokenizer *t, const gchar *content_type)
-{
-	struct _HTMLTokenizerPrivate *p;
-	if (!is_text (content_type))
-		return;
-
-	p = t->priv;
-
-	if (!p->enableconvert)
-		return;
-
-	if (p->content_type)
-		g_free(p->content_type);
-
-	p->content_type = g_ascii_strdown ( content_type, -1);
-
-	if (is_valid_g_iconv (p->iconv_cd))
-		g_iconv_close (p->iconv_cd);
-
-	p->iconv_cd = generate_iconv_from (p->content_type);
-
-#if 0
-	if (charset_is_utf8 (p->content_type))
-		g_warning ("Trying UTF-8");
-	else
-		g_warning ("Trying %s",p->content_type);
-#endif
-}
-
-static void
-html_tokenizer_real_begin (HTMLTokenizer *t, const gchar *content_type)
+html_tokenizer_real_begin (HTMLTokenizer *t, gchar *content_type)
 {
 	struct _HTMLTokenizerPrivate *p = t->priv;
 
@@ -781,8 +490,17 @@ html_tokenizer_real_begin (HTMLTokenizer *t, const gchar *content_type)
 	p->searchCount = 0;
 	p->searchGtkHTMLCount = 0;
 	p->title = FALSE;
+	p->charEntity = FALSE;
+
+	p->utf8 = charset_is_utf8 (content_type);
+	p->utf8_length = 0;
+#if 0
+	if (p->utf8)
+		g_warning ("Trying UTF-8");
+	else
+		g_warning ("Trying ISO-8859-1");
+#endif
 
-	html_tokenizer_real_change (t, content_type);
 }
 
 static void
@@ -843,25 +561,6 @@ html_tokenizer_append_token (HTMLTokenizer *t, const gchar *string, gint len)
 	}
 }
 
-static void
-add_byte (HTMLTokenizer *t, const gchar **c)
-{
-	add_char (t,**c);
-	(*c) ++;
-}
-
-static void
-add_char (HTMLTokenizer *t, gchar c)
-{
-	struct _HTMLTokenizerPrivate *p = t->priv;
-	if (c!='\0')
-	{
-		*(p->dest) = c;
-		p->dest++;
-		*(p->dest) = 0;
-	}
-}
-
 static void
 html_tokenizer_append_token_buffer (HTMLTokenizer *t, gint min_size)
 {
@@ -893,31 +592,31 @@ html_tokenizer_add_pending (HTMLTokenizer *t)
 	struct _HTMLTokenizerPrivate *p = t->priv;
 
 	if (p->tag || p->select) {
-		add_char (t, ' ');
+		add_unichar (t, ' ');
 	}
 	else if (p->textarea) {
 		if (p->pending == LFPending)
-			add_char (t, '\n');
+			add_unichar (t, '\n');
 		else
-			add_char (t, ' ');
+			add_unichar (t, ' ');
 	}
 	else if (p->pre) {
 		switch (p->pending) {
 		case SpacePending:
-			add_char (t, ' ');
+			add_unichar (t, ' ');
 			break;
 		case LFPending:
 			if (p->dest > p->buffer) {
 				html_tokenizer_append_token (t, p->buffer, p->dest - p->buffer);
 			}
 			p->dest = p->buffer;
-			add_char (t, TAG_ESCAPE);
-			add_char (t, '\n');
+			add_unichar (t, TAG_ESCAPE);
+			add_unichar (t, '\n');
 			html_tokenizer_append_token (t, p->buffer, 2);
 			p->dest = p->buffer;
 			break;
 		case TabPending:
-			add_char (t, '\t');
+			add_unichar (t, '\t');
 			break;
 		default:
 			g_warning ("Unknown pending type: %d\n", (gint) p->pending);
@@ -925,7 +624,7 @@ html_tokenizer_add_pending (HTMLTokenizer *t)
 		}
 	}
 	else {
-		add_char (t, ' ');
+		add_unichar (t, ' ');
 	}
 
 	p->pending = NonePending;
@@ -950,12 +649,12 @@ in_comment (HTMLTokenizer *t, const gchar **src)
 {
 	struct _HTMLTokenizerPrivate *p = t->priv;
 
-	if (**src == '-') {		     /* Look for "-->" */
+	if (**src == '-') {	             /* Look for "-->" */
 		if (p->searchCount < 2)
 			p->searchCount++;
 	} else if (p->searchCount == 2 && (**src == '>')) {
 		p->comment = FALSE;          /* We've got a "-->" sequence */
-	} else if (tolower (**src) == gtkhtmlStart[p->searchGtkHTMLCount]) {
+	} else if (tolower (**src) == gtkhtmlStart [p->searchGtkHTMLCount]) {
 		if (p->searchGtkHTMLCount == 8) {
 			p->extension    = TRUE;
 			p->comment = FALSE;
@@ -963,7 +662,7 @@ in_comment (HTMLTokenizer *t, const gchar **src)
 			p->searchExtensionEndCount = 0;
 			p->searchGtkHTMLCount = 0;
 		} else
-			p->searchGtkHTMLCount++;
+			p->searchGtkHTMLCount ++;
 	} else {
 		p->searchGtkHTMLCount = 0;
 		if (p->searchCount < 2)
@@ -991,7 +690,7 @@ in_extension (HTMLTokenizer *t, const gchar **src)
 	/* check for "-->" */
 	if (!p->tquote && **src == '-') {
 		if (p->searchExtensionEndCount < 2)
-			p->searchExtensionEndCount++;
+			p->searchExtensionEndCount ++;
 		(*src) ++;
 	} else if (!p->tquote && p->searchExtensionEndCount == 2 && **src == '>') {
 		p->extension = FALSE;
@@ -1003,7 +702,7 @@ in_extension (HTMLTokenizer *t, const gchar **src)
 
 				while (p->searchExtensionEndCount) {
 					extension_one_char (t, &c);
-					p->searchExtensionEndCount--;
+					p->searchExtensionEndCount --;
 				}
 			}
 		}
@@ -1020,10 +719,10 @@ in_script_or_style (HTMLTokenizer *t, const gchar **src)
 	if (p->scriptCodeSize + 11 > p->scriptCodeMaxSize)
 		p->scriptCode = g_realloc (p->scriptCode, p->scriptCodeMaxSize += 1024);
 
-	if ((**src == '>' ) && ( p->searchFor[p->searchCount] == '>')) {
+	if ((**src == '>' ) && ( p->searchFor [p->searchCount] == '>')) {
 		(*src)++;
-		p->scriptCode[p->scriptCodeSize] = 0;
-		p->scriptCode[p->scriptCodeSize + 1] = 0;
+		p->scriptCode [p->scriptCodeSize] = 0;
+		p->scriptCode [p->scriptCodeSize + 1] = 0;
 		if (p->script) {
 			p->script = FALSE;
 		}
@@ -1036,12 +735,12 @@ in_script_or_style (HTMLTokenizer *t, const gchar **src)
 	/* Check if a </script> tag is on its way */
 	else if (p->searchCount > 0) {
 		gboolean put_to_script = FALSE;
-		if (tolower (**src) == p->searchFor[p->searchCount]) {
-			p->searchBuffer[p->searchCount] = **src;
+		if (tolower (**src) == p->searchFor [p->searchCount]) {
+			p->searchBuffer [p->searchCount] = **src;
 			p->searchCount++;
 			(*src)++;
 		}
-		else if (p->searchFor[p->searchCount] == '>') {
+		else if (p->searchFor [p->searchCount] == '>') {
 			/* There can be any number of white-space characters between
 			   tag name and closing '>' so try to move through them, if possible */
 
@@ -1049,36 +748,227 @@ in_script_or_style (HTMLTokenizer *t, const gchar **src)
 			while (isspace (**p))
 				(*p)++;
 
+			
 			if (**p == '>')
 				*src = *p;
 			else
 				put_to_script = TRUE;
 		}
-		else
+		else 
 			put_to_script = TRUE;
 
 		if (put_to_script) {
 			gchar *c;
 
-			p->searchBuffer[p->searchCount] = 0;
+			p->searchBuffer [p->searchCount] = 0;
 			c = p->searchBuffer;
 			while (*c)
-				p->scriptCode[p->scriptCodeSize++] = *c++;
-			p->scriptCode[p->scriptCodeSize] = **src; (*src)++;
+				p->scriptCode [p->scriptCodeSize++] = *c++;
+			p->scriptCode [p->scriptCodeSize] = **src; (*src)++;
 			p->searchCount = 0;
 		}
 	}
 	else if (**src == '<') {
 		p->searchCount = 1;
-		p->searchBuffer[0] = '<';
+		p->searchBuffer [0] = '<';
 		(*src)++;
 	}
 	else {
-		p->scriptCode[p->scriptCodeSize] = **src;
+		p->scriptCode [p->scriptCodeSize] = **src;
 		(*src)++;
 	}
 }
 
+static gunichar win1252_to_unicode [32] = {
+	0x20ac,
+	0x81,
+	0x201a,
+	0x0192,
+	0x201e,
+	0x2026,
+	0x2020,
+	0x2021,
+	0x02c6,
+	0x2030,
+	0x0160,
+	0x2039,
+	0x0152,
+	0x8d,
+	0x017d,
+	0x8f,
+	0x90,
+	0x2018,
+	0x2019,
+	0x201c,
+	0x201d,
+	0x2022,
+	0x2013,
+	0x2014,
+	0x02dc,
+	0x2122,
+	0x0161,
+	0x203a,
+	0x0153,
+	0x9d,
+	0x017e,
+	0x0178
+};
+
+static void
+add_unichar (HTMLTokenizer *t, gunichar wc)
+{
+	struct _HTMLTokenizerPrivate *p = t->priv;
+
+	p->utf8_length = 0;
+
+	/*
+	  chars in range 128 - 159 are control characters in unicode,
+	  but most browsers treat them as windows 1252
+	  encoded characters and translate them to unicode
+	  it's broken, but we do the same here
+	*/
+	if (wc > 127 && wc < 160)
+		wc = win1252_to_unicode [wc - 128];
+
+	if (wc != '\0') {
+		p->dest += g_unichar_to_utf8 (wc, p->dest);
+		*(p->dest) = 0;
+	}
+}
+
+static void
+add_byte (HTMLTokenizer *t, const gchar **src)
+{
+	gunichar wc;
+	struct _HTMLTokenizerPrivate *p = t->priv;
+
+	if (p->utf8) {
+		p->utf8_buffer[p->utf8_length] = **src;
+		p->utf8_length++;
+
+		wc = g_utf8_get_char_validated ((const gchar *)p->utf8_buffer, p->utf8_length);
+		if (wc == -1 || p->utf8_length >= (sizeof(p->utf8_buffer)/sizeof(p->utf8_buffer[0]))) {
+			add_unichar (t, INVALID_CHARACTER_MARKER);
+			(*src)++;
+			return;
+		} else if (wc == -2) {
+			/* incomplete character check again */
+			(*src)++;
+			return;
+		}
+	} else {
+		wc = (guchar)**src;
+	}
+
+	add_unichar (t, wc);
+	(*src)++;
+}
+
+static void
+flush_entity (HTMLTokenizer *t)
+{
+	struct _HTMLTokenizerPrivate *p = t->priv;
+	/* ignore the TAG_ESCAPE when flushing */
+	const char *str = p->searchBuffer + 1;
+
+	 while (p->searchCount--) {
+		add_byte (t, &str);
+	}
+}
+
+static gboolean
+add_unichar_validated (HTMLTokenizer *t, gunichar uc)
+{
+	if (g_unichar_validate (uc)) {
+		add_unichar (t, uc);
+		return TRUE;
+	}
+
+	g_warning ("invalid character value: x%xd", uc);
+	return FALSE;
+}
+
+static void
+in_entity (HTMLTokenizer *t, const gchar **src)
+{
+	struct _HTMLTokenizerPrivate *p = t->priv;
+	gunichar entityValue = 0;
+
+	/* See http://www.mozilla.org/newlayout/testcases/layout/entities.html for a complete entity list,
+	   ftp://ftp.unicode.org/Public/MAPPINGS/ISO8859/8859-1.TXT
+	   (or 'man iso_8859_1') for the character encodings. */
+
+	p->searchBuffer [p->searchCount + 1] = **src;
+	p->searchBuffer [p->searchCount + 2] = '\0';
+
+	/* Check for &#0000 sequence */
+	if (p->searchBuffer[2] == '#') {
+		if ((p->searchCount > 1) &&
+		    (!isdigit (**src)) &&
+		    (p->searchBuffer[3] != 'x')) {
+			/* &#123 */
+			p->searchBuffer [p->searchCount + 1] = '\0';
+			entityValue = strtoul (&(p->searchBuffer [3]),
+					       NULL, 10);
+			p->charEntity = FALSE;
+		}
+		if ((p->searchCount > 1) &&
+		    (!isalnum (**src)) &&
+		    (p->searchBuffer[3] == 'x')) {
+			/* &x12AB */
+			p->searchBuffer [p->searchCount + 1] = '\0';
+
+			entityValue = strtoul (&(p->searchBuffer [4]),
+					       NULL, 16);
+			p->charEntity = FALSE;
+		}
+	}
+	else {
+		/* Check for &abc12 sequence */
+		if (!isalnum (**src)) {
+			p->charEntity = FALSE;
+			if ((p->searchBuffer [p->searchCount + 1] == ';') ||
+			    (!p->tag)) {
+				char *ename = p->searchBuffer + 2;
+
+				p->searchBuffer [p->searchCount + 1] = '\0'; /* FIXME sucks */
+				entityValue = html_entity_parse (ename, 0);
+			}
+		}
+
+	}
+
+	if (p->searchCount > 13) {
+		/* Ignore this sequence since it's too long */
+		p->charEntity = FALSE;
+		flush_entity (t);
+	}
+	else if (p->charEntity) {
+				/* Keep searching for end of character entity */
+		p->searchCount++;
+		(*src)++;
+	}
+	else {
+		/*
+		 * my reading of http://www.w3.org/TR/html4/intro/sgmltut.html#h-3.2.2 makes
+		 * seem correct to always collapse entity references, even in element names
+		 * and attributes.
+		 */
+		if (entityValue) {
+			if (entityValue != TAG_ESCAPE)
+				/* make sure the entity value is a valid character value */
+				if (!add_unichar_validated (t, entityValue))
+					add_unichar (t, INVALID_CHARACTER_MARKER);
+
+			if (**src == ';')
+				(*src)++;
+		} else {
+			/* Ignore the sequence, just add it as plaintext */
+			flush_entity (t);
+		}
+	}
+}
+
 static void
 in_tag (HTMLTokenizer *t, const gchar **src)
 {
@@ -1104,7 +994,7 @@ in_tag (HTMLTokenizer *t, const gchar **src)
 				/* Invalid tag, just add it */
 		if (p->pending)
 			html_tokenizer_add_pending (t);
-		add_char (t, '<');
+		add_unichar (t, '<');
 		add_byte (t, src);
 		return;
 	}
@@ -1116,12 +1006,30 @@ in_tag (HTMLTokenizer *t, const gchar **src)
 		html_tokenizer_append_token (t, p->buffer, p->dest - p->buffer);
 		p->dest = p->buffer;
 	}
-	add_char (t, TAG_ESCAPE);
-	add_char (t, '<');
+	add_unichar (t, TAG_ESCAPE);
+	add_unichar (t, '<');
 	p->tag = TRUE;
 	p->searchCount = 1; /* Look for <!-- to start comment */
 }
 
+static void
+start_entity (HTMLTokenizer *t, const gchar **src)
+{
+	struct _HTMLTokenizerPrivate *p = t->priv;
+
+	(*src)++;
+
+	p->discard = NoneDiscard;
+
+	if (p->pending)
+		html_tokenizer_add_pending (t);
+
+	p->charEntity      = TRUE;
+	p->searchBuffer[0] = TAG_ESCAPE;
+	p->searchBuffer[1] = '&';
+	p->searchCount     = 1;
+}
+
 static void
 start_tag (HTMLTokenizer *t, const gchar **src)
 {
@@ -1138,12 +1046,12 @@ end_tag (HTMLTokenizer *t, const gchar **src)
 
 	p->searchCount = 0; /* Stop looking for <!-- sequence */
 
-	add_char (t, '>');
+	add_unichar (t, '>');
 
 	/* Make the tag lower case */
 	ptr = p->buffer + 2;
 	if (p->pre || *ptr == '/') {
-		/* End tag */
+	    	/* End tag */
 		p->discard = NoneDiscard;
 	}
 	else {
@@ -1160,6 +1068,7 @@ end_tag (HTMLTokenizer *t, const gchar **src)
 	p->dest = p->buffer;
 
 	p->tag = FALSE;
+	p->aTag = FALSE;
 	p->pending = NonePending;
 	(*src)++;
 
@@ -1262,6 +1171,8 @@ in_crlf (HTMLTokenizer *t, const gchar **src)
 static void
 in_space_or_tab (HTMLTokenizer *t, const gchar **src)
 {
+	gchar *ptr;
+
 	if (t->priv->tquote) {
 		if (t->priv->discard == NoneDiscard)
 			t->priv->pending = SpacePending;
@@ -1270,6 +1181,9 @@ in_space_or_tab (HTMLTokenizer *t, const gchar **src)
 		t->priv->searchCount = 0; /* Stop looking for <!-- sequence */
 		if (t->priv->discard == NoneDiscard)
 			t->priv->pending = SpacePending;
+		ptr = t->priv->buffer;
+		if (strlen (ptr) == 3 && ptr[1] == '<' && ptr[2] == 'a')
+			t->priv->aTag = TRUE;
 	}
 	else if (t->priv->pre || t->priv->textarea) {
 		if (t->priv->pending)
@@ -1294,7 +1208,7 @@ in_quoted (HTMLTokenizer *t, const gchar **src)
 		t->priv->searchCount = 0; /* Stop looking for <!-- sequence */
 		if ((t->priv->tquote == SINGLE_QUOTE && **src == '\"') /* match " */
 		    || (t->priv->tquote == DOUBLE_QUOTE && **src == '\'')) {
-			add_char (t, **src);
+			add_unichar (t, **src);
 			(*src)++;
 		} else if (*(t->priv->dest-1) == '=' && !t->priv->tquote) {
 			t->priv->discard = SpaceDiscard;
@@ -1304,7 +1218,7 @@ in_quoted (HTMLTokenizer *t, const gchar **src)
 				t->priv->tquote = DOUBLE_QUOTE;
 			else
 				t->priv->tquote = SINGLE_QUOTE;
-			add_char (t, **src);
+			add_unichar (t, **src);
 			(*src)++;
 		}
 		else if (t->priv->tquote) {
@@ -1331,7 +1245,7 @@ in_assignment (HTMLTokenizer *t, const gchar **src)
 	t->priv->discard = NoneDiscard;
 	if (t->priv->tag) {
 		t->priv->searchCount = 0; /* Stop looking for <!-- sequence */
-		add_char (t, '=');
+		add_unichar (t, '=');
 		if (!t->priv->tquote) {
 			t->priv->pending = NonePending;
 			t->priv->discard = SpaceDiscard;
@@ -1341,7 +1255,7 @@ in_assignment (HTMLTokenizer *t, const gchar **src)
 		if (t->priv->pending)
 			html_tokenizer_add_pending (t);
 
-		add_char (t, '=');
+		add_unichar (t, '=');
 	}
 	(*src)++;
 }
@@ -1395,8 +1309,12 @@ html_tokenizer_tokenize_one_char (HTMLTokenizer *t, const gchar **src)
 		in_extension (t, src);
 	else if (p->script || p->style)
 		in_script_or_style (t, src);
+	else if (p->charEntity)
+		in_entity (t, src);
 	else if (p->startTag)
 		in_tag (t, src);
+	else if (**src == '&' && !p->aTag)
+		start_entity (t, src);
 	else if (**src == '<' && !p->tag)
 		start_tag (t, src);
 	else if (**src == '>' && p->tag && !p->tquote)
@@ -1414,7 +1332,7 @@ html_tokenizer_tokenize_one_char (HTMLTokenizer *t, const gchar **src)
 }
 
 static void
-html_tokenizer_real_write (HTMLTokenizer *t, const gchar *string, gsize size)
+html_tokenizer_real_write (HTMLTokenizer *t, const gchar *string, size_t size)
 {
 	const gchar *src = string;
 
@@ -1422,7 +1340,7 @@ html_tokenizer_real_write (HTMLTokenizer *t, const gchar *string, gsize size)
 		html_tokenizer_tokenize_one_char (t, &src);
 }
 
-static const gchar *
+static gchar *
 html_tokenizer_blocking_get_name (HTMLTokenizer *t)
 {
 	switch (GPOINTER_TO_INT (t->priv->blocking->data)) {
@@ -1463,28 +1381,11 @@ html_tokenizer_blocking_pop (HTMLTokenizer *t)
 /** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
 
 void
-html_tokenizer_begin (HTMLTokenizer *t, const gchar *content_type)
-{
-
-	g_return_if_fail (t && HTML_IS_TOKENIZER (t));
-
-	g_signal_emit (t, html_tokenizer_signals[HTML_TOKENIZER_BEGIN_SIGNAL], 0, content_type);
-}
-
-void
-html_tokenizer_set_engine_type (HTMLTokenizer *t, gboolean engine_type)
+html_tokenizer_begin (HTMLTokenizer *t, gchar *content_type)
 {
 	g_return_if_fail (t && HTML_IS_TOKENIZER (t));
 
-	g_signal_emit (t, html_tokenizer_signals[HTML_TOKENIZER_CHANGEENGINE_SIGNAL], 0, engine_type);
-}
-
-void
-html_tokenizer_change_content_type (HTMLTokenizer *t,const gchar *content_type)
-{
-	g_return_if_fail (t && HTML_IS_TOKENIZER (t));
-
-	g_signal_emit (t, html_tokenizer_signals[HTML_TOKENIZER_CHANGECONTENT_SIGNAL], 0, content_type);
+	g_signal_emit (t, html_tokenizer_signals [HTML_TOKENIZER_BEGIN_SIGNAL], 0, content_type);
 }
 
 void
@@ -1496,7 +1397,7 @@ html_tokenizer_end (HTMLTokenizer *t)
 }
 
 void
-html_tokenizer_write (HTMLTokenizer *t, const gchar *str, gsize size)
+html_tokenizer_write (HTMLTokenizer *t, const gchar *str, size_t size)
 {
 	HTMLTokenizerClass *klass;
 
@@ -1526,39 +1427,6 @@ html_tokenizer_peek_token (HTMLTokenizer *t)
 
 }
 
-const gchar *
-html_tokenizer_get_content_type(HTMLTokenizer *t)
-{
-	HTMLTokenizerClass *klass;
-
-	g_return_val_if_fail (t && HTML_IS_TOKENIZER (t), NULL);
-
-	klass = HTML_TOKENIZER_CLASS (G_OBJECT_GET_CLASS (t));
-
-	if (klass->get_content_type)
-		return  klass->get_content_type(t);
-
-	g_warning ("No get_content_type method defined.");
-	return NULL;
-
-}
-
-gboolean
-html_tokenizer_get_engine_type (HTMLTokenizer *t)
-{
-	HTMLTokenizerClass *klass;
-
-	g_return_val_if_fail (t && HTML_IS_TOKENIZER (t),FALSE);
-
-	klass = HTML_TOKENIZER_CLASS (G_OBJECT_GET_CLASS (t));
-
-	if (klass->get_engine_type)
-		return  klass->get_engine_type(t);
-
-	g_warning ("No get_engine_type method defined.");
-	return FALSE;
-}
-
 gchar *
 html_tokenizer_next_token (HTMLTokenizer *t)
 {
